#!/usr/bin/perl
use POSIX strftime;

# Plugin defaults
my $VERSION       = "v2";
my $DEFAULT_GROUP = "Networking";
my $MODULE_GROUP  = "Networking";
my $MODULE_TAGS   = undef; # defined in conf file
my $HELP = "Pandora FMS SNMP plugin $VERSION\n"
		. "Please call this script as described next:\n"
		. "\npandora_aruba.pl <basic_parameters> [-N <modules_name>] [-G <modules_group>] [<v3_parameters>]"
		. "\n\n"
		. "<basic_parameters>: \n"
		. "\t-a <agent_name> \n"
		. "\t-v <version> \n"
		. "\t-c <community> \n"
		. "\t-h <target_host> \n"
		. "\t-o <oid_base> \n"
		. "\t-n <names_tree> \n"
		. "\t-s <status_tree> \n"
		. "\n"
		. "<v3_parameters>: \n"
		. "\t-P <privProtocol> \n"
		. "\t-A <authProtocol> \n"
		. "\t-C <context> \n"
		. "\t-K <privKey> \n"
		. "\t-L <securityLevel> \n"
		. "\t-k <authKey> \n"
		. "\n\nExample:"
		. "\npandora_aruba.pl -a agent1 -v 1 -c public -h 10.10.10.10 -o .1.3.6.1.2.1.2.2.1 -n .2 -s .3"
		. "\n\n";


# Variables
my %config;
my %params;
my $xmlstatus;
my $copy;
my $del;
my $timestamp = strftime '%Y-%m-%d %H:%M:%S', localtime;
my @snmp_objects;

%params = ( 
$ARGV[0] => $ARGV[1], 
$ARGV[2] => $ARGV[3], 
$ARGV[4] => $ARGV[5], 
$ARGV[6] => $ARGV[7], 
$ARGV[8] => $ARGV[9], 
$ARGV[10] => $ARGV[11], 
$ARGV[12] => $ARGV[13], 
$ARGV[14] => $ARGV[15], 
$ARGV[16] => $ARGV[17], 
$ARGV[18] => $ARGV[19], 
$ARGV[20] => $ARGV[21], 
$ARGV[22] => $ARGV[23], 
$ARGV[24] => $ARGV[25], 
$ARGV[26] => $ARGV[27], 
$ARGV[28] => $ARGV[29], 
$ARGV[30] => $ARGV[31], 
$ARGV[32] => $ARGV[33] 
);

my ($sec,$min,$hour,$mday,$mon,$year)=localtime(time);
$year += 1900;
$mon += 1;
my $fichero=$params{-a}.".".$year.$mon.$mday.$hour.$min.$sec.".data";
open FICHERO_DESTINO, ">/tmp/$fichero";

$copy = "cp /tmp/".$fichero." /var/spool/pandora/data_in/ && rm -f /tmp/".$fichero;
$del = "rm -f /tmp/".$fichero;

###########################################
#
# Pandora FMS Plugin functions library
#
#
# (c) Fco de Borja Sanchez <fborja.sanchez@artica.es>
#
# Modified by Enrique Martin Garcia <enrique.martin@artica.es>
#
# Version   Date
#  a2        17-05-2016
# 
###########################################################


####################################
###### COMMON FUNCTIONS BEGIN ######
####################################

####
# Erase blank spaces before and after the string 
####################################################
sub trim($){
	my $string = shift;
	if (empty ($string)){
		return "";
	}
	$string =~ s/^\s+//;
	$string =~ s/"//g;
	$string =~ s/\s+$//;
	return $string;
}

#####
# Empty
###################################
sub empty($){
	my $str = shift;

	if (! defined $str){
		return 1;
	}
	if (! $str){
		return 1;
	}
	if ($str =~ /^\ *[\n\r]{0,2}$/) {
		return 1;
	}
	return 0;
}

#####
## Get unit
#############################
sub getUnit($){
	my $str = shift;
	$str =~ s/[\d\.\,]//g;
	return $str;
}

#####
# print_module
###################################
sub print_module ($){
	my $data = shift;
	
	# If not a string type, remove all blank spaces!    
	if ($data->{type} !~ m/string/){
		$data->{value} = trim($data->{value});
	}

	$data->{tags}  = $data->{tags}?$data->{tags}:($config{MODULE_TAG_LIST}?$config{MODULE_TAG_LIST}:undef);
	$data->{interval}     = $data->{interval}?$data->{interval}:($config{MODULE_INTERVAL}?$config{MODULE_INTERVAL}:undef);
	$data->{module_group} = $data->{module_group}?$data->{module_group}:($config{MODULE_GROUP}?$config{MODULE_GROUP}:$MODULE_GROUP);

	print FICHERO_DESTINO "<module>\n";
	print FICHERO_DESTINO "\t<name><![CDATA[" . $data->{NAME} . "]]></name>\n";
	print FICHERO_DESTINO "\t<type>" . $data->{type} . "</type>\n";
	print FICHERO_DESTINO "\t<data><![CDATA[" . $data->{value} . "]]></data>\n";
	print FICHERO_DESTINO "\t<description><![CDATA[" . $data->{desc} . "]]></description>\n";
	if ( !(empty ($data->{unit})) ) {
		print FICHERO_DESTINO "\t<unit><![CDATA[" . $data->{unit} . "]]></unit>\n";
	}
	if (! (empty($data->{interval})) ) {
		print FICHERO_DESTINO "\t<interval><![CDATA[" . $data->{interval} . "]]></interval>\n";
	}
	if (! (empty($data->{tags})) ) {
		print FICHERO_DESTINO "\t<tags>" . $data->{tags} . "></tags>\n";
	}
	if (! (empty($data->{module_group})) ) {
		print FICHERO_DESTINO "\t<module_group>" . $data->{module_group} . "</module_group>\n";
	}
	if (! (empty($data->{wmin})) ) {
		print FICHERO_DESTINO "\t<min_warning><![CDATA[" . $data->{wmin} . "]]></min_warning>\n";
	}
	if (! (empty($data->{wmax})) ) {
		print FICHERO_DESTINO "\t<max_warning><![CDATA[" . $data->{wmax} . "]]></max_warning>\n";
	}
	if (! (empty ($data->{cmin})) ) {
		print FICHERO_DESTINO "\t<min_critical><![CDATA[" . $data->{cmin} . "]]></min_critical>\n";
	}
	if (! (empty ($data->{cmax})) ){
		print FICHERO_DESTINO "\t<max_critical><![CDATA[" . $data->{cmax} . "]]></max_critical>\n";
	}

	print FICHERO_DESTINO "</module>\n";

}

#####
## Module warning
##      - tag: name
##      - value: severity (default 0)
##      - msg: description of the message
###########################################
sub print_warning($$;$){
	my ($tag, $msg, $value) = @_;

	if (!(isEnabled($config{informational_modules}))) {
		return 0;
	}

	if (!(isEnabled($config{informational_monitors}))) {
		$value = 0;
	}

	my %module;
	$module{NAME}  = "Plugin message" . ( $tag?" " . $tag:"");
	$module{type}  = "generic_data";
	$module{value} = defined($value)?$value:0;
	$module{desc}  = $msg;
	$module{wmin}  = 1;
	$module{cmin}  = 3;
	print_module(\%module);
}


#####
## Plugin devolution in case of error
#######################################
sub print_error($){
	my $msg = shift;
	my %module;
	$module{NAME}  = "Plugin execution error";
	$module{type}  = "generic_proc";
	$module{value} = 0;
	$module{desc}  = $msg;
	print_module(\%module);
	exit -1;
}

#####
## Log data
########################
sub logger ($$) {
	my ($tag, $message) = @_;

	my $file = $config{log};
	defined $file or $file = $GLOBAL_LOG_FILE;

	# Log rotation
	if (-e $file && (stat($file))[7] > 32000) {
		rename ($file, $file.'.old');
	}
	if ($log_aux_flag == 0) {
		# Log starts
		if (! open (FILE, "> $file")) {
			print_error "[ERROR] Could not open logfile '$file' \n";
		}
		$log_aux_flag = 1;
	}
	else {
		if (! open (FILE, ">> $file")) {
			print_error "[ERROR] Could not open logfile '$file' \n";
		}
	}

	$message = "[" . $tag . "] " . $message if ((defined $tag) && ($tag ne ""));

	print FILE strftime ("%Y-%m-%d %H:%M:%S", localtime()) . " - " . $message . "\n";
	close (FILE);
}


#####
## is Enabled 
#################
sub isEnabled($){
	my $value = shift;
	
	if ((defined ($value)) && ($value > 0)){
		# return true
		return 1;
	}
	#return false
	return 0;

}


#####
## General configuration file parser
##
## log=/PATH/TO/LOG/FILE
##
#######################################
sub parse_configuration($){
	my $conf_file = shift;
	my %config;

	open (FILE,"<", "$conf_file") or return undef;

	while (<FILE>){
		if (($_ =~ /^ *$/)
		 || ($_ =~ /^#/ )){
		 	# skip blank lines and comments
			next;
		}
		my @parsed = split /=/, $_, 2;
		$config{trim($parsed[0])} = trim($parsed[1]);
	}
	close (FILE);

	return %config;
}

##################################
###### COMMON FUNCTIONS END ######
##################################

###
# SNMP walk value
#  will return the snmpwalk output
#
#  $community (v1,2,2c)
#   -> means $context (v3)
# 
#  Configuration hash
# 
# 	$snmp{version}
# 	$snmp{community}
# 	$snmp{host}
# 	$snmp{oid}
# 	$snmp{port}
#   $snmp{securityName}
#   $snmp{context
#   $snmp{securityLevel}
#   $snmp{authProtocol}
#   $snmp{authKey}
#   $snmp{privProtocol}
#   $snmp{privKey}
####################################################
sub snmp_walk($) {
	my $snmp = shift;
	my $cmd;

	if ( defined ($snmp->{version} )
	  && (($snmp->{version} eq "1")
	   || ($snmp->{version} eq "2")
	   || ($snmp->{version} eq "2c"))) {

		if (defined $snmp->{port}){
			$cmd = "snmpwalk -On -v $snmp->{version} -c $snmp->{community} $snmp->{host}:$snmp->{port} $snmp->{oid}";
		}
		else {
			$cmd = "snmpwalk -On -v $snmp->{version} -c $snmp->{community} $snmp->{host} $snmp->{oid}";
		}

	}
	elsif ( defined ($snmp->{version} )
	  && ($snmp->{version} eq "3") ) { # SNMP v3
		# Authentication required

		# $securityLevel = (noAuthNoPriv|authNoPriv|authPriv);

		# unauthenticated request
		# Ex. snmpwalk -On -v 3 -n "" -u noAuthUser -l noAuthNoPriv test.net-snmp.org sysUpTime

		# authenticated request
		# Ex. snmpwalk -On -v 3 -n "" -u MD5User -a MD5 -A "The Net-SNMP Demo Password" -l authNoPriv test.net-snmp.org sysUpTime

		# authenticated and encrypted request
		# Ex. snmpwalk -On -v 3 -n "" -u MD5DESUser -a MD5 -A "The Net-SNMP Demo Password" -x DES -X "The Net-SNMP Demo Password" -l authPriv test.net-snmp.org system

		if ($snmp->{securityLevel} =~ /^noAuthNoPriv$/i){
			# Unauthenticated request

			if (defined $snmp->{port}){
				$cmd = "snmpwalk -On -v $snmp->{version} -n \"$snmp->{context}\" -l $snmp->{securityLevel} $snmp->{host}:$snmp->{port} $snmp->{oid}";
			}
			else {
				$cmd = "snmpwalk -On -v $snmp->{version} -n \"$snmp->{context}\" -l $snmp->{securityLevel} $snmp->{host} $snmp->{oid}";
			}
		}
		elsif ($snmp->{securityLevel} =~ /^authNoPriv$/i){ 
			# Authenticated request

			if (defined $snmp->{port}){
				$cmd = "snmpwalk -On -v $snmp->{version} -n \"$snmp->{context}\" -u $snmp->{securityName} -a $snmp->{authProtocol} -A $snmp->{authKey} -l $snmp->{securityLevel} $snmp->{host}:$snmp->{port} $snmp->{oid}";
			}
			else {
				$cmd = "snmpwalk -On -v $snmp->{version} -n \"$snmp->{context}\" -u $snmp->{securityName} -a $snmp->{authProtocol} -A $snmp->{authKey} -l $snmp->{securityLevel} $snmp->{host} $snmp->{oid}";
			}
		}
		elsif ($snmp->{securityLevel} =~ /^authPriv$/i){
			# Authenticated and encrypted request

			if (defined $snmp->{port}){
				$cmd = "snmpwalk -On -v $snmp->{version} -n \"$snmp->{context}\" -u $snmp->{securityName} -l $snmp->{securityLevel} -a $snmp->{authProtocol} -A $snmp->{authKey} -x $snmp->{privProtocol} -X $snmp->{privKey} $snmp->{host}:$snmp->{port} $snmp->{oid}";
			}
			else {
				$cmd = "snmpwalk -On -v $snmp->{version} -n \"$snmp->{context}\" -u $snmp->{securityName} -l $snmp->{securityLevel} -a $snmp->{authProtocol} -A $snmp->{authKey} -x $snmp->{privProtocol} -X $snmp->{privKey} $snmp->{host} $snmp->{oid}";
			}
		}
	}
	else {
		print STDERR "Only SNMP 1 2 2c and 3 are supported.\n";
		exit -1;
	}

	my $result = `$cmd`;
	if ($? != 0){
		return undef;
	}
	return $result;

}



###
# SNMP get value
#  will return a hash with the data and datatype
#
#  $community (v1,2,2c)
#   -> means $context (v3)
# 
#  Configuration hash
# 
# 	$snmp{version}
# 	$snmp{community}
# 	$snmp{host}
# 	$snmp{oid}
# 	$snmp{port}
#   $snmp{securityName}
#   $snmp{context
#   $snmp{securityLevel}
#   $snmp{authProtocol}
#   $snmp{authKey}
#   $snmp{privProtocol}
#   $snmp{privKey}
####################################################
sub snmp_get($) {
	my $snmp = shift;
	my $cmd;

	if ( defined ($snmp->{version} )
	  && (($snmp->{version} eq "1")
	   || ($snmp->{version} eq "2")
	   || ($snmp->{version} eq "2c"))) {

		if (defined $snmp->{port}){
			$cmd = "snmpget -On -v $snmp->{version} -c $snmp->{community} $snmp->{host}:$snmp->{port} $snmp->{oid}";
		}
		else {
			$cmd = "snmpget -On -v $snmp->{version} -c $snmp->{community} $snmp->{host} $snmp->{oid}";
		}

	}
	elsif ( defined ($snmp->{version} )
	  && ($snmp->{version} eq "3") ) { # SNMP v3
		# Authentication required

		# $securityLevel = (noAuthNoPriv|authNoPriv|authPriv);

		# unauthenticated request
		# Ex. snmpget -On -v 3 -n "" -u noAuthUser -l noAuthNoPriv test.net-snmp.org sysUpTime

		# authenticated request
		# Ex. snmpget -On -v 3 -n "" -u MD5User -a MD5 -A "The Net-SNMP Demo Password" -l authNoPriv test.net-snmp.org sysUpTime

		# authenticated and encrypted request
		# Ex. snmpget -On -v 3 -n "" -u MD5DESUser -a MD5 -A "The Net-SNMP Demo Password" -x DES -X "The Net-SNMP Demo Password" -l authPriv test.net-snmp.org system

		if ($snmp->{securityLevel} =~ /^noAuthNoPriv$/i){
			# Unauthenticated request

			if (defined $snmp->{port}){
				$cmd = "snmpget -On -v $snmp->{version} -n \"$snmp->{context}\" -l $snmp->{securityLevel} $snmp->{host}:$snmp->{port} $snmp->{oid}";
			}
			else {
				$cmd = "snmpget -On -v $snmp->{version} -n \"$snmp->{context}\" -l $snmp->{securityLevel} $snmp->{host} $snmp->{oid}";
			}
		}
		elsif ($snmp->{securityLevel} =~ /^authNoPriv$/i){ 
			# Authenticated request

			if (defined $snmp->{port}){
				$cmd = "snmpget -On -v $snmp->{version} -n \"$snmp->{context}\" -u $snmp->{securityName} -a $snmp->{authProtocol} -A $snmp->{authKey} -l $snmp->{securityLevel} $snmp->{host}:$snmp->{port} $snmp->{oid}";
			}
			else {
				$cmd = "snmpget -On -v $snmp->{version} -n \"$snmp->{context}\" -u $snmp->{securityName} -a $snmp->{authProtocol} -A $snmp->{authKey} -l $snmp->{securityLevel} $snmp->{host} $snmp->{oid}";
			}
		}
		elsif ($snmp->{securityLevel} =~ /^authPriv$/i){
			# Authenticated and encrypted request

			if (defined $snmp->{port}){
				$cmd = "snmpget -On -v $snmp->{version} -n \"$snmp->{context}\" -u $snmp->{securityName} -l $snmp->{securityLevel} -a $snmp->{authProtocol} -A $snmp->{authKey} -x $snmp->{privProtocol} -X $snmp->{privKey} $snmp->{host}:$snmp->{port} $snmp->{oid}";
			}
			else {
				$cmd = "snmpget -On -v $snmp->{version} -n \"$snmp->{context}\" -u $snmp->{securityName} -l $snmp->{securityLevel} -a $snmp->{authProtocol} -A $snmp->{authKey} -x $snmp->{privProtocol} -X $snmp->{privKey} $snmp->{host} $snmp->{oid}";
			}
		}
	}
	else {
		print STDERR "Only SNMP 1 2 2c and 3 are supported.\n";
		exit -1;
	}

	my $result = `$cmd`;
	if ($? != 0){
		return undef;
	}
	return snmp_data_switcher((split /=\ /, $result)[1]);

}



#####
## returns a hash with [type->datatype][data->value]
######################################################
sub snmp_data_switcher ($){
	my @st_data = split /\: /, $_[0];
	my %data;

	my $pure_data = trim($st_data[1]) or undef;
	$data{data} = $pure_data;
	
	if ( uc($st_data[0]) eq uc("INTEGER")) {
		$data{type} = "generic_data";
	}
	elsif (uc($st_data[0]) eq uc("Integer32")) {
		$data{type} = "generic_data";
	}
	elsif (uc($st_data[0]) eq uc("octect string")) {
		$data{type} = "generic_data";
	}
	elsif (uc($st_data[0]) eq uc("bits")) {
		$data{type} = "generic_data";
	}
	elsif (uc($st_data[0]) eq uc("object identifier")) {
		$data{type} = "generic_data_string";
	}
	elsif (uc($st_data[0]) eq uc("IpAddress")) {
		$data{type} = "generic_data_string";
	}
	elsif (uc($st_data[0]) eq uc("Counter")) {
		$data{type} = "generic_data_inc";
	}
	elsif (uc($st_data[0]) eq uc("Counter32")) {
		$data{type} = "generic_data_inc";
	}
	elsif (uc($st_data[0]) eq uc("Gauge")) {
		$data{type} = "generic_data";
	}
	elsif (uc($st_data[0]) eq uc("Unsigned32")) {
		$data{type} = "generic_data_inc";
	}
	elsif (uc($st_data[0]) eq uc("TimeTicks")) {
		$data{type} = "generic_data_string";
	}
	elsif (uc($st_data[0]) eq uc("Opaque")) {
		$data{type} = "generic_data_string";
	}
	elsif (uc($st_data[0]) eq uc("Counter64")) {
		$data{type} = "generic_data_inc";
	}
	elsif (uc($st_data[0]) eq uc("UInteger32")) {
		$data{type} = "generic_data";
	}
	elsif (uc($st_data[0]) eq uc("BIT STRING")) {
		$data{type} = "generic_data_string";
	}
	elsif (uc($st_data[0]) eq uc("STRING")) {
		$data{type} = "generic_data_string";
	}
	else {
		$data{type} = "generic_data_string";
	}

	if ($data{type} eq "generic_data"){
		($data{data} = $pure_data) =~ s/\D*//g;
	}

	return \%data;
}


#####
## prepare oid
#######################
sub prepare_oid($$;@) {
	my ($snmp_item, $field,@xtra) = @_;
	$snmp_item->{oid} = $snmp_item->{oid_base} . $snmp_item->{$field};
	foreach my $i (@xtra){
		$snmp_item->{oid} .= $i;
	}
	return \%{$snmp_item};
}

#####
##
#######################
sub get_item_names(){
	my $snmp_item = prepare_oid(\%config, "names_tree");

	my @result = split /\n/, snmp_walk($snmp_item);
	foreach my $line (@result){
		my %item;
		my ($id,$value) = (split /=\ /, (split(/$snmp_item->{oid}/, $line, 2))[1], 2);

		$item{name} = snmp_data_switcher($value);
		$item{large_uid} = trim($id);
		$item{large_uid} =~ /\ /;
		push @snmp_objects, \%item;
	}
}


#####
## get and print modules
##########################
sub get_item_xtra($$){
	my ($field,$tag) = @_;
	print FICHERO_DESTINO "<agent_data description='Agent generated by pandora_aruba.pl' group='Unknown' os_name='Other' os_version='' interval='300' version='' timestamp='$timestamp' agent_name='$params{-a}' timezone_offset='0'>\n";
	foreach my $item (@snmp_objects){
		my %snmp_item = %config;

		$snmp_item = prepare_oid(\%config, $field, $item->{large_uid});
		my $value = snmp_get($snmp_item);

		my %module;
		
		$module{NAME}  = "$item->{name}{data} $tag";
		$module{type}  = $value->{type};
		$module{value} = $value->{data};

		print_module(\%module);
	}
	print FICHERO_DESTINO "</agent_data>";
}




###############################
##
#----> MAIN
##
####################################################

if ($#ARGV < 13){
	print STDERR $HELP;
	print_error "Not enough arguments in call.";
}

#%config = parse_configuration($ARGV[0]);
%config = ( 
privProtocol => "$params{-P}", 
oid_base => "$params{-o}", 
names_tree => "$params{-n}", 
status_tree => "$params{-s}", 
version => "$params{-v}", 
port => "161", 
host => "$params{-h}", 
securityName => "", 
community => "$params{-c}", 
MODULE_GROUP => "$params{-G}", 
authProtocol => "$params{-A}", 
context => "$params{-C}", 
privKey => "$params{-K}", 
securityLevel => "$params{-L}", 
authKey => "$params{-k}" 
);

get_item_names();
get_item_xtra("status_tree", "$params{-N}");

$xmlstatus = qx(xmllint /tmp/$fichero | wc -l);

if ($xmlstatus != 0){
		system("echo 1");
		exec ($copy);
	}
	else{
		system("echo 0");
		exec ($del);
	}
	